@@ -683,10 +683,31 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         self._vmutils._jobutils.add_virt_resource.assert_called_once_with(
             mock_nic, mock_vm)
 
+    def test_get_nic_data_by_name(self):
+        nic_cls = self._vmutils._conn.Msvm_SyntheticEthernetPortSettingData
+        nic_cls.return_value = [mock.sentinel.nic]
+
+        nic = self._vmutils._get_nic_data_by_name(mock.sentinel.name)
+
+        self.assertEqual(mock.sentinel.nic, nic)
+        nic_cls.assert_called_once_with(ElementName=mock.sentinel.name)
+
+    def test_get_missing_nic_data_by_name(self):
+        nic_cls = self._vmutils._conn.Msvm_SyntheticEthernetPortSettingData
+        nic_cls.return_value = []
+        self.assertRaises(
+            exceptions.HyperVvNicNotFound,
+            self._vmutils._get_nic_data_by_name,
+            mock.sentinel.name)
+
     @mock.patch.object(vmutils.VMUtils, '_get_nic_data_by_name')
     def test_destroy_nic(self, mock_get_nic_data_by_name):
         mock_nic_data = mock_get_nic_data_by_name.return_value
 
+        # We expect this exception to be ignored.
+        self._vmutils._jobutils.remove_virt_resource.side_effect = (
+            exceptions.NotFound(message='fake_exc'))
+
         self._vmutils.destroy_nic(self._FAKE_VM_NAME,
                                   mock.sentinel.FAKE_NIC_NAME)
 
@@ -865,20 +886,26 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         mock_disk = self._prepare_mock_disk()
 
         self._vmutils.detach_vm_disk(self._FAKE_VM_NAME,
-                                     self._FAKE_HOST_RESOURCE)
+                                     self._FAKE_HOST_RESOURCE,
+                                     serial=mock.sentinel.serial)
         self._vmutils._jobutils.remove_virt_resource.assert_called_once_with(
             mock_disk)
 
-    def test_get_mounted_disk_resource_from_path(self):
-        mock_disk_1 = mock.MagicMock()
-        mock_disk_2 = mock.MagicMock()
-        mock_disk_2.HostResource = [self._FAKE_MOUNTED_DISK_PATH]
-        self._vmutils._conn.query.return_value = [mock_disk_1, mock_disk_2]
+    @ddt.data(None, mock.sentinel.serial)
+    def test_get_mounted_disk_resource_from_path(self, serial):
+        mock_disk = mock.MagicMock()
+
+        if serial:
+            self._vmutils._conn.query.return_value = [mock_disk]
+        else:
+            mock_disk.HostResource = [self._FAKE_MOUNTED_DISK_PATH]
+            self._vmutils._conn.query.return_value = [
+                mock.MagicMock(), mock_disk]
 
         physical_disk = self._vmutils._get_mounted_disk_resource_from_path(
-            self._FAKE_MOUNTED_DISK_PATH, True)
+            self._FAKE_MOUNTED_DISK_PATH, True, serial=serial)
 
-        self.assertEqual(mock_disk_2, physical_disk)
+        self.assertEqual(mock_disk, physical_disk)
 
     def test_get_controller_volume_paths(self):
         self._prepare_mock_disk()
@@ -1267,64 +1294,44 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
                 vs_data, mock.sentinel.certificate_required)
             mock_modify_virtual_system.assert_called_once_with(vs_data)
 
-    @mock.patch.object(vmutils.VMUtils,
-                       '_get_mounted_disk_resource_from_path')
-    def test_set_disk_qos_specs(self, mock_get_disk_resource):
-        mock_disk = mock_get_disk_resource.return_value
+    def test_set_disk_qos_specs_exc(self):
+        self.assertRaises(exceptions.UnsupportedOperation,
+                          self._vmutils.set_disk_qos_specs,
+                          mock.sentinel.disk_path, mock.sentinel.max_iops)
 
-        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path,
-                                         max_iops=mock.sentinel.max_iops,
-                                         min_iops=mock.sentinel.min_iops)
+    def test_set_disk_qos_specs_noop(self):
+        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path, 0, 0)
 
-        mock_get_disk_resource.assert_called_once_with(
-            mock.sentinel.disk_path, is_physical=False)
-        self.assertEqual(mock.sentinel.max_iops, mock_disk.IOPSLimit)
-        self.assertEqual(mock.sentinel.min_iops, mock_disk.IOPSReservation)
-        self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
-            mock_disk)
-
-    @mock.patch.object(vmutils.VMUtils,
-                       '_get_mounted_disk_resource_from_path')
-    def test_set_disk_qos_specs_missing_values(self, mock_get_disk_resource):
-        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path)
-
-        self.assertFalse(mock_get_disk_resource.called)
-
-    def _test_is_drive_physical(self, is_physical):
-        self._vmutils._pathutils.exists.return_value = not is_physical
-        ret = self._vmutils._is_drive_physical(mock.sentinel.fake_drive_path)
-
-        self.assertEqual(is_physical, ret)
-
-    def test_is_drive_phyisical_true(self):
-        self._test_is_drive_physical(is_physical=True)
-
-    def test_is_drive_physical_false(self):
-        self._test_is_drive_physical(is_physical=False)
-
-    @mock.patch.object(vmutils.VMUtils, '_is_drive_physical')
+    @ddt.data(
+        {'drive_path':
+            r'\\ADCONTROLLER\root\virtualization\v2:Msvm_DiskDrive.'
+            r'CreationClassName="Msvm_DiskDrive",DeviceID="Microsoft:'
+            r'6344C73D-6FD6-4A74-8BE8-8EEAC2737369\\0\\0\\D",'
+            r'SystemCreationClassName="Msvm_ComputerSystem"',
+         'exp_phys_disk': True},
+        {'drive_path': 'some_image.vhdx',
+         'exp_phys_disk': False})
+    @ddt.unpack
     @mock.patch.object(vmutils.VMUtils,
                        '_get_mounted_disk_resource_from_path')
     def test_drive_to_boot_source(self, mock_get_disk_res_from_path,
-                                  mock_is_drive_physical):
-        mock_is_drive_physical.return_value = True
+                                  drive_path, exp_phys_disk):
         mock_drive = mock.MagicMock()
         mock_drive.Parent = mock.sentinel.bssd
         mock_get_disk_res_from_path.return_value = mock_drive
 
-        mock_rasd_path = mock_drive.path_.return_value
+        exp_rasd_path = (mock_drive.path_.return_value
+                         if exp_phys_disk else mock_drive.Parent)
         mock_same_element = mock.MagicMock()
         self._vmutils._conn.Msvm_LogicalIdentity.return_value = [
             mock.Mock(SameElement=mock_same_element)]
 
-        ret = self._vmutils._drive_to_boot_source(mock.sentinel.drive_path)
+        ret = self._vmutils._drive_to_boot_source(drive_path)
 
         self._vmutils._conn.Msvm_LogicalIdentity.assert_called_once_with(
-            SystemElement=mock_rasd_path)
-        mock_is_drive_physical.assert_called_once_with(
-            mock.sentinel.drive_path)
+            SystemElement=exp_rasd_path)
         mock_get_disk_res_from_path.assert_called_once_with(
-            mock.sentinel.drive_path, is_physical=True)
+            drive_path, is_physical=exp_phys_disk)
         expected_path = mock_same_element.path_.return_value
         self.assertEqual(expected_path, ret)
 
@@ -1558,3 +1565,35 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
 
     def test_remove_all_pci_devices(self):
         self._vmutils.remove_all_pci_devices(mock.sentinel.vm_name)
+
+
+class VMUtils6_3TestCase(test_base.OsWinBaseTestCase):
+
+    def setUp(self):
+        super(VMUtils6_3TestCase, self).setUp()
+        self._vmutils = vmutils.VMUtils6_3()
+        self._vmutils._conn_attr = mock.MagicMock()
+        self._vmutils._jobutils = mock.MagicMock()
+
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    def test_set_disk_qos_specs(self, mock_get_disk_resource):
+        mock_disk = mock_get_disk_resource.return_value
+
+        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path,
+                                         max_iops=mock.sentinel.max_iops,
+                                         min_iops=mock.sentinel.min_iops)
+
+        mock_get_disk_resource.assert_called_once_with(
+            mock.sentinel.disk_path, is_physical=False)
+        self.assertEqual(mock.sentinel.max_iops, mock_disk.IOPSLimit)
+        self.assertEqual(mock.sentinel.min_iops, mock_disk.IOPSReservation)
+        self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
+            mock_disk)
+
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    def test_set_disk_qos_specs_missing_values(self, mock_get_disk_resource):
+        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path)
+
+        self.assertFalse(mock_get_disk_resource.called)
