@@ -621,8 +621,12 @@ class VMUtils(baseutils.BaseUtilsVirt):
                          'address': address})
 
     def _get_nic_data_by_name(self, name):
-        return self._conn.Msvm_SyntheticEthernetPortSettingData(
-            ElementName=name)[0]
+        nics = self._conn.Msvm_SyntheticEthernetPortSettingData(
+            ElementName=name)
+        if nics:
+            return nics[0]
+
+        raise exceptions.HyperVvNicNotFound(vnic_name=name)
 
     def create_nic(self, vm_name, nic_name, mac_address):
         """Create a (synthetic) nic and attach it to the vm."""
@@ -648,8 +652,13 @@ class VMUtils(baseutils.BaseUtilsVirt):
         :param nic_name: The NIC's ElementName.
         """
         # TODO(claudiub): remove vm_name argument, no longer used.
-        nic_data = self._get_nic_data_by_name(nic_name)
-        self._jobutils.remove_virt_resource(nic_data)
+        try:
+            nic_data = self._get_nic_data_by_name(nic_name)
+            self._jobutils.remove_virt_resource(nic_data)
+        except exceptions.NotFound:
+            LOG.debug("Ignoring NotFound exception while attempting "
+                      "to remove vm nic: '%s'. It may have been already "
+                      "deleted.", nic_name)
 
     def soft_shutdown_vm(self, vm_name):
         vm = self._lookup_vm_check(vm_name, as_vssd=False)
@@ -1054,28 +1063,30 @@ class VMUtils(baseutils.BaseUtilsVirt):
                   'this Hyper-V version.'))
 
     def set_disk_qos_specs(self, disk_path, max_iops=None, min_iops=None):
-        if min_iops is None and max_iops is None:
-            LOG.debug("Skipping setting disk QoS specs as no "
-                      "value was provided.")
-            return
+        """Hyper-V disk QoS policy.
 
-        disk_resource = self._get_mounted_disk_resource_from_path(
-            disk_path, is_physical=False)
+        This feature is supported on Windows / Hyper-V Server 2012 R2 or newer.
 
-        if max_iops is not None:
-            disk_resource.IOPSLimit = max_iops
-        if min_iops is not None:
-            disk_resource.IOPSReservation = min_iops
-
-        self._jobutils.modify_virt_resource(disk_resource)
-
-    def _is_drive_physical(self, drive_path):
-        # TODO(atuvenie): Find better way to check if path represents
-        # physical or virtual drive.
-        return not self._pathutils.exists(drive_path)
+        :raises os_win.exceptions.UnsupportedOperation: if the given max_iops
+            or min_iops have non-zero values.
+        """
+        if min_iops or max_iops:
+            raise exceptions.UnsupportedOperation(
+                reason=_("Virtual disk QoS is not supported on this "
+                         "hypervisor version."))
 
     def _drive_to_boot_source(self, drive_path):
-        is_physical = self._is_drive_physical(drive_path)
+        # We expect the drive path to be the one that was passed to the
+        # 'attach_drive' or 'attach_volume_to_controller' methods. In case of
+        # passthrough disks, the drive path will be a Msvm_DiskDrive WMI
+        # object path while for image files it will be the actual image path.
+        #
+        # Note that Msvm_DiskDrive objects will also exist for attached disk
+        # images, but that's not what we'll get in this situation. If we ever
+        # need to accept Msvm_DiskDrive object paths for image files as well,
+        # an extra check will be needed, but that may lead to some other
+        # inconsistencies.
+        is_physical = r'root\virtualization\v2:Msvm_DiskDrive' in drive_path
         drive = self._get_mounted_disk_resource_from_path(
             drive_path, is_physical=is_physical)
 
@@ -1264,3 +1275,23 @@ class VMUtils(baseutils.BaseUtilsVirt):
     def _set_vm_snapshot_type(self, vmsettings, snapshot_type):
         # Supported on Windows Server 2016 or newer.
         pass
+
+
+class VMUtils6_3(VMUtils):
+
+    def set_disk_qos_specs(self, disk_path, max_iops=None, min_iops=None):
+        """Sets the disk's QoS policy."""
+        if min_iops is None and max_iops is None:
+            LOG.debug("Skipping setting disk QoS specs as no "
+                      "value was provided.")
+            return
+
+        disk_resource = self._get_mounted_disk_resource_from_path(
+            disk_path, is_physical=False)
+
+        if max_iops is not None:
+            disk_resource.IOPSLimit = max_iops
+        if min_iops is not None:
+            disk_resource.IOPSReservation = min_iops
+
+        self._jobutils.modify_virt_resource(disk_resource)
